!----------------------------------------------------------------------
! Nota Bene : maintenant le save se fait dans mathematica
!----------------------------------------------------------------------
!   Wunderlich (divisé par u2) mars 2022
!   pour moebius vary Lk
!   avec des functions psi diverses
!   _pas_ de terme de Healey (en 1/2 eps eta'^2)
!   _mais_ un terme eps eta'' apres divsion par u2 et mult par 1+eta^2
!   s in (0;0.5)
!   s=0 au flip
!   s=0.5 au point de wunder
!   wunderlich-moebius-2020-v02-conti.nb
!----------------------------------------------------------------------
! 218 pas en 3.5sec (ndim=18, ncol=5, ntst=50)
!----------------------------------------------------------------------
! w = 0.1/pi : 1/w = 31.4159265
! w = 0.2/pi : 1/w = 15.707963
! w = 0.5/pi : 1/w = 6.2831853
!
!! la loi de AUTO : NICP + NDIM = NBC + NINT + 1

INCLUDE 'routines.f'

!======================================================================
DOUBLE PRECISION FUNCTION GETPSI(w,veta,typepsi)
! z = w * eta'
! psi(z) = [ log(2+z) - log(2-z) ] / z
!======================================================================
INTEGER, INTENT(IN) :: typepsi
DOUBLE PRECISION, INTENT(IN) :: w, veta
double precision x
!----------------------------------------------------------------------
x = w * veta ! noté z dans les calculs de couche limite
GETPSI = 0.d0

if (typepsi.eq.10) then
  GETPSI = 1.d0 + (x**2)/12.d0 + (x**4)/80.d0 + (x**6)/448.d0 &
           + (x**8)/2304.d0 + (x**10)/11264.d0
  GETPSI = GETPSI + 0.05d0 / (4.d0 - x**2)
endif

if (typepsi.eq.-1) then ! fonction pure, avec dev lim en x=0
  ! par defaut, on met le dev lim
  GETPSI = 1.d0 + (x**2)/12.d0 + (x**4)/80.d0 + (x**6)/448.d0 &
           + (x**8)/2304.d0 + (x**10)/11264.d0

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETPSI = ( dLog(2.d0 + x) - dLog(2.d0 - x) ) / x
  endif
endif

if (typepsi.eq.-2) then ! fonction pure, avec dev lim en x=0, et truc pour |x|>2
  ! par defaut, on met le dev lim
  GETPSI = 1.d0 + (x**2)/12.d0 + (x**4)/80.d0 + (x**6)/448.d0 &
           + (x**8)/2304.d0 + (x**10)/11264.d0

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETPSI = ( dLog(2.d0 + x) - dLog(2.d0 - x) ) / x
  endif

  if (dabs(x).ge.2.d0) then ! si hors de la zone, c-a-d si |x|> 2
    GETPSI = 1.d0 + (x**2)/12.d0 + (x**4)/80.d0 + (x**6)/448.d0 &
             + (x**8)/2304.d0 + (x**10)/11264.d0
    GETPSI = GETPSI + 0.05d0 / (4.d0 - x**2)
  endif
endif

if (dabs(GETPSI).gt.1e16) then
  print'(A,I4,A,ES11.4,A,ES11.4)'," (typepsi = ",typepsi, " Fonction trop grande : psi(",x,") = ",GETPSI
endif

END FUNCTION GETPSI
!======================================================================


!======================================================================
DOUBLE PRECISION FUNCTION GETdPSI(w,veta,typepsi)
! z = w * eta'
! psi(z) = [ log(2+z) - log(2-z) ] / z
!======================================================================
INTEGER, INTENT(IN) :: typepsi
DOUBLE PRECISION, INTENT(IN) :: w, veta
double precision x
!----------------------------------------------------------------------
x = w * veta ! noté z dans les calculs de couche limite
GETdPSI = 0.d0

if (typepsi.eq.10) then
  GETdPSI = x/6.d0 + (x**3)/20.d0 + 3.d0/224.d0*x**5 &
             + (x**7)/288.d0 + 5.d0/5632.d0*x**9
  GETdPSI = GETdPSI + 0.1d0*x/(4.d0 - x**2)**2
endif

if (typepsi.eq.-1) then ! fonction pure, avec dev lim en x=0
  ! par defaut, on met le dev lim
  GETdPSI = x/6.d0 + (x**3)/20.d0 + 3.d0/224.d0*x**5 &
             + (x**7)/288.d0 + 5.d0/5632.d0*x**9

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETdPSI =  1.d0 / ((2.d0 - x)*x) + 1.d0 / (x*(2.d0 + x)) &
                  + dLog(2.d0 - x) / x**2 - dLog(2.d0 + x) / x**2
  endif
endif

if (typepsi.eq.-2) then ! fonction pure, avec dev lim en x=0, et truc pour |x|>2
  ! par defaut, on met le dev lim
  GETdPSI = x/6.d0 + (x**3)/20.d0 + 3.d0/224.d0*x**5 &
             + (x**7)/288.d0 + 5.d0/5632.d0*x**9

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETdPSI =  1.d0 / ((2.d0 - x)*x) + 1.d0 / (x*(2.d0 + x)) &
                  + dLog(2.d0 - x) / x**2 - dLog(2.d0 + x) / x**2
  endif

  if (dabs(x).ge.2d0) then ! si hors de la zone, c-a-d si |x|> 2
    GETdPSI = x/6.d0 + (x**3)/20.d0 + 3.d0/224.d0*x**5 &
               + (x**7)/288.d0 + 5.d0/5632.d0*x**9
    GETdPSI = GETdPSI + 0.1d0*x/(4.d0 - x**2)**2
  endif
endif

if (dabs(GETdPSI).gt.1e16) then
  print'(A,I4,A,ES11.4,A,ES11.4)'," (typepsi = ",typepsi, " Derivee trop grande : dpsi(",x,") = ",GETdPSI
endif

END FUNCTION GETdPSI
!======================================================================


!======================================================================
DOUBLE PRECISION FUNCTION GETddPSI(w,veta,typepsi)
! z = w * eta'
! psi(z) = [ log(2+z) - log(2-z) ] / z
!======================================================================
INTEGER, INTENT(IN) :: typepsi
DOUBLE PRECISION, INTENT(IN) :: w, veta
double precision x
!----------------------------------------------------------------------
x = w * veta ! noté z dans les calculs de couche limite
if (dabs(x).gt.2.d0) then
  !print'(A,F7.3)',"bla",x ! A veut dire string
  !print'(A,ES11.4,A,ES11.4)'," Hors zone : |x|>2 : x = ",x
endif

GETddPSI = 0.d0

if (typepsi.eq.10) then
  GETddPSI = 1/6.d0 + 3.d0/20.d0*x**2 + 15.d0/224.d0*x**4 &
             + 7.d0/288.d0*x**6 + 45.d0/5632.d0*x**8
  GETddPSI = GETddPSI + 0.05d0 * ( (8.d0 + 6.d0*x**2) / (4.d0 - x**2)**3 )
endif

if (typepsi.eq.-1) then ! fonction pure, avec dev lim en x=0
  ! par defaut, on met le dev lim
  GETddPSI = 1/6.d0 + 3.d0/20.d0*x**2 + 15.d0/224.d0*x**4 &
             + 7.d0/288.d0*x**6 + 45.d0/5632.d0*x**8

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETddPSI =  8.d0 * x * (-2.d0 + x**2) / (-4.d0 + x**2)**2 &
               - dLog(2.d0 - x) + dLog(2.d0 + x)
    GETddPSI = 2 * GETddPSI / x**3
  endif
endif

if (typepsi.eq.-2) then ! fonction pure, avec dev lim en x=0, et truc pour |x|>2
  ! par defaut, on met le dev lim
  GETddPSI = 1/6.d0 + 3.d0/20.d0*x**2 + 15.d0/224.d0*x**4 &
             + 7.d0/288.d0*x**6 + 45.d0/5632.d0*x**8

  if (dabs(x).gt.0.005d0) then ! si hors de la zone de comp. cata.
    GETddPSI =  8.d0 * x * (-2.d0 + x**2) / (-4.d0 + x**2)**2 &
               - dLog(2.d0 - x) + dLog(2.d0 + x)
    GETddPSI = 2 * GETddPSI / x**3
  endif

  if (dabs(x).ge.2d0) then ! si hors de la zone, c-a-d si |x|> 2
    GETddPSI = 1/6.d0 + 3.d0/20.d0*x**2 + 15.d0/224.d0*x**4 &
               + 7.d0/288.d0*x**6 + 45.d0/5632.d0*x**8
    GETddPSI = GETddPSI + 0.05d0 * ( (8.d0 + 6.d0*x**2) / (4.d0 - x**2)**3 )
  endif
endif

if (dabs(GETddPSI).gt.1e16) then
  if ((dabs(x)-2.d0).gt.1e-06) then ! si on est 'loin' de +-2
    print'(A,I4,A,ES17.7,A,ES11.3)'," (typepsi = ",typepsi, " Courbure trop grande : ddpsi(",x,") = ",GETddPSI
  endif
endif

END FUNCTION GETddPSI
!======================================================================




!======================================================================
SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
!======================================================================

IMPLICIT NONE
INTEGER, INTENT(IN) :: NDIM, IJAC, ICP(*)
DOUBLE PRECISION, INTENT(IN) :: U(NDIM), PAR(*)
DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
DOUBLE PRECISION, INTENT(INOUT) :: DFDU(NDIM,*), DFDP(NDIM,*)

double precision m1,m2,m3,x,y,z,d1x,d1y,d1z,d2x,d2y,d2z,d3x,d3y,d3z,u1,u2,u3
double precision nx,ny,nz,m1p,m2p,m3p,n1,n2,n3,smax,eta,veta,twrate
double precision psi,dpsi,ddpsi
double precision num,den,Pa,Pb,Qa,Qb
integer i
! getpar()
double precision n20,n30,m20,m30,eta0,w,alphaL,Lk,Tw,Wr,eps ! a recopier
integer typeconti,typepsi,typeF18    ! a re copier
! fin getpar()
DOUBLE PRECISION, EXTERNAL :: getpsi, getdpsi, getddpsi
!----------------------------------------------------------------------

! --- On lit les PAR
CALL GETPAR(n20,n30,m20,m30,eta0,w,typepsi,typeconti,alphaL,Lk,Tw,Wr,eps,typeF18,PAR)
! FIN On lit les PAR

m1  =   U(01)
m2  =   U(02)
m3  =   U(03)
x   =   U(04)
y   =   U(05)
z   =   U(06)
d1x =   U(07)
d1y =   U(08)
d1z =   U(09)
d2x =   U(10)
d2y =   U(11)
d2z =   U(12)
d3x =   U(13)
d3y =   U(14)
d3z =   U(15)
twrate  =   U(16)
eta =   U(17)
veta=   U(18)  ! veta = eta'  (z = w eta')

psi = getpsi(w,veta,typepsi) ! psi(w eta')
dpsi = getdpsi(w,veta,typepsi) ! psi'(z) ou z = w eta'
ddpsi = getddpsi(w,veta,typepsi) ! psi''(z) ou z = w eta'

u1  = 0.d0
u2  = (m2 + eta * m3) / (1.d0 + eta**2)**2 / psi
u3  = eta * u2


nx = 0.d0 ! d123 alignes avec exyz en s=0
ny = n20
nz = n30 ! pas de gravite

n1 = nx*d1x + ny*d1y + nz*d1z
n2 = nx*d2x + ny*d2y + nz*d2z
n3 = nx*d3x + ny*d3y + nz*d3z

m1p =  n2 - m3 * u2 + m2 * u3
m2p = -n1 - m1 * u3 + m3 * u1
m3p =     - m2 * u1 + m1 * u2

F(01) =  m1p  ! m1'(s)
F(02) =  m2p  ! m2'(s)
F(03) =  m3p  ! m3'(s)

F(04) =  d3x  ! x'
F(05) =  d3y  ! y'
F(06) =  d3z  ! z'

F(07) = -d3x * u2 + d2x * u3
F(08) = -d3y * u2 + d2y * u3
F(09) = -d3z * u2 + d2z * u3
F(10) = -d1x * u3 + d3x * u1
F(11) = -d1y * u3 + d3y * u1
F(12) = -d1z * u3 + d3z * u1
F(13) = -d2x * u1 + d1x * u2
F(14) = -d2y * u1 + d1y * u2
F(15) = -d2z * u1 + d1z * u2

F(16) = u3  ! tw' = u3
F(17) = veta ! eta' = veta

! --- Calcul de F18
Pa = psi**2 / (ddpsi*psi - 2.d0*dpsi**2)
Pb = psi*dpsi / (ddpsi*psi - 2.d0*dpsi**2)

if (typeF18.eq.1) then ! a utiliser quand ???
  Qa = Pa / (m2 + m3*eta)
  if (dabs(Qa).gt.1e12) then
    print'(A,ES11.4)'," (typeF18=1) Qa trop grand : Qa = ",Qa
  endif

  Qb = Pb / (m2 + m3*eta)
  if (dabs(Qb).gt.1e12) then
    print'(A,ES11.4)'," (typeF18=1) Qb trop grand : Qb = ",Qb
  endif

  num = Qa * (2 * m2 * eta + m3 * eta**2 - m3)
  num = num + 2 * eta * w * veta * Pb
  num = num + (1+eta**2) * (n1*w - m3*w*veta) * Qb
  den = 0.5d0*w**2 * (1+eta**2) + eps * Qa
  F(18) = num/den ! veta' = ...
endif

if (typeF18.eq.3) then ! epsilon_mh = 0 vraiment
  Qa = Pa / (m2 + m3*eta)
  if (dabs(Qa).gt.1e12) then
    print'(A,ES11.4)'," (typeF18=1) Qa trop grand : Qa = ",Qa
  endif

  Qb = Pb / (m2 + m3*eta)
  if (dabs(Qb).gt.1e12) then
    print'(A,ES11.4)'," (typeF18=1) Qb trop grand : Qb = ",Qb
  endif

  num = Qa * (2 * m2 * eta + m3 * eta**2 - m3)
  num = num + 2 * eta * w * veta * Pb
  num = num + (1+eta**2) * (n1*w - m3*w*veta) * Qb
  den = 0.5d0*w**2 * (1+eta**2) ! ZERO vraiment + eps * Qa
  F(18) = num/den ! veta' = ...
endif

if (typeF18.eq.2) then ! a utiliser quand y a du epsilon_mh
  num = Pa * (2 * m2 * eta + m3 * eta**2 - m3)
  num = num + 2 * eta * (m2 + m3*eta) * w * veta * Pb
  num = num + (1+eta**2) * (n1*w - m3*w*veta) * Pb
  den = 0.5d0*w**2 * (1+eta**2) * (m2 + m3*eta) + eps * Pa
  if (dabs(den).lt.1e-14) then
    print'(A,ES11.4)'," (typeF18=2) denominateur trop petit : den = ",den
  endif
  F(18) = num/den ! veta' = ...
endif

! FIN Calcul de F18

smax = 0.5d0  ! la longueur totale dans auto doit faire 1
DO i = 1, 18, 1
  F(i) = F(i) * smax
END DO

END SUBROUTINE FUNC
!======================================================================





!======================================================================
SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
!======================================================================

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: NDIM, ICP(*), NBC, IJAC
    DOUBLE PRECISION, INTENT(IN) :: PAR(*), U0(NDIM), U1(NDIM)
    DOUBLE PRECISION, INTENT(OUT) :: FB(NBC)
    DOUBLE PRECISION, INTENT(INOUT) :: DBC(NBC,*)

    ! getpar()
    double precision n20,n30,m20,m30,eta0,w,alphaL,Lk,Tw,Wr,eps ! a recopier
    integer typeconti,typepsi,typeF18    ! a re copier
    ! fin getpar()

    double precision temp
    double precision d1M,d1P,d3P
    dimension d1M(3),d1P(3),d3P(3)
!----------------------------------------------------------------------

! --- On lit les PAR
CALL GETPAR(n20,n30,m20,m30,eta0,w,typepsi,typeconti,alphaL,Lk,Tw,Wr,eps,typeF18,PAR)
! FIN On lit les PAR

! --- Initial conditions (at s=0)
FB(01)=U0(01) - 0.d0 ! m1(0)
FB(02)=U0(02) - m20   ! m2(0)
FB(03)=U0(03) - m30   ! m3(0)

FB(04)=U0(04) - 0.d0   ! x(0)=0
FB(05)=U0(05) - 0.d0   ! y(0)=0
FB(06)=U0(06) - 0.d0   ! z(0)=0

FB(07)=U0(07) - 1.d0   ! d1x(0)
FB(08)=U0(08) - 0.d0   ! d1y(0)
FB(09)=U0(09) - 0.d0   ! d1z(0)

FB(10)=U0(10) - 0.d0   ! d2x(0)
FB(11)=U0(11) - 1.d0   ! d2y(0)
FB(12)=U0(12) - 0.d0   ! d2z(0)

FB(13)=U0(13) - 0.d0   ! d3x(0)
FB(14)=U0(14) - 0.d0   ! d3y(0)
FB(15)=U0(15) - 1.d0   ! d3z(0)

FB(16)=U0(16) - 0.d0   ! tw(0)=0
FB(17)=U0(17) - eta0   ! eta(0)=eta0
FB(18)=U0(18) - 0.d0   ! eta'(0)=0
! FIN Initial conditions (at s=0)

! --- Final conditions (at s=1/2)
FB(19)=U1(13) - 0.d0   ! d3x(1/2)=0
FB(20)=U1(05) - 0.d0   ! y(1/2)=0
FB(21)=U1(06) - 0.d0   ! z(1/2)=0
FB(22)=U1(17) - 0.d0   ! eta(1/2)=0
! FIN Final conditions (at s=1/2)

if (typeconti.eq.0) then ! vary Lk (fixed : eps, ...)
    ! rien a faire
endif

if (typeconti.eq.1) then ! vary w ou eps (fixed : Lk, ...)
  ! --- calcul de alpha_here
  d1M(1)= U1(07)
  d1M(2)=-U1(08)   ! d1(-1/2) (on utilise la symetrie)
  d1M(3)=-U1(09)

  d1P(1)=U1(07)
  d1P(2)=U1(08)   ! d1(+1/2)
  d1P(3)=U1(09)

  d3P(1)=U1(13)
  d3P(2)=U1(14)   ! d3(+1/2)
  d3P(3)=U1(15)

  call COMPAL(d1M,d1P,d3P,temp,1)  ! le resultat est dans 'temp'
  ! FIN calcul de alpha here
  FB(23)=alphaL - temp      ! alpha=alphaCOMP
endif

if (typeconti.eq.2) then ! vary (w ou eps) at Lk=1/2
  FB(23)=U1(07)-0.d0  ! d1x(1/2) = 0
endif

END SUBROUTINE BCND
!======================================================================





!======================================================================
SUBROUTINE GETPAR(n20,n30,m20,m30,eta0,w,typepsi,typeconti,alphaL,Lk,Tw,Wr,eps,typeF18,PAR)
!======================================================================
!   READ PARAMETERS

implicit none
! getpar()
double precision n20,n30,m20,m30,eta0,w,alphaL,Lk,Tw,Wr,eps ! a recopier
integer typeconti,typepsi,typeF18    ! a re copier
! fin getpar()

double precision PAR(*)
double precision pi
!----------------------------------------------------------------------
pi=4*dATan(1.d0)

n20      =   PAR(01)
n30      =   PAR(02)
m20      =   PAR(03)
m30      =   PAR(04)
eta0     =   PAR(05)
w        =   1.d0 / PAR(06)
typepsi  =   PAR(07)
typeconti=   PAR(08)
alphaL   =   PAR(09)
Lk       =   PAR(10)
Tw       =   PAR(11)
Wr       =   PAR(12)
eps      =   1.d0 / PAR(23)
typeF18  =   PAR(24)

END SUBROUTINE GETPAR
!======================================================================



!======================================================================
SUBROUTINE PVLS(NDIM,U,PAR,numli)   ! pour les post-calculated param
!======================================================================

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: NDIM,numli
    DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
    DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)
    DOUBLE PRECISION, EXTERNAL :: GETP,GETU2
    INTEGER NDX,NCOL,NTST

    ! getpar()
    double precision n20,n30,m20,m30,eta0,w,alphaL,Lk,Tw,Wr,eps ! a recopier
    integer typeconti,typepsi,typeF18    ! a re copier
    ! fin getpar()

    double precision alphaH
    double precision d1M,d1P,d3P
    dimension d1M(3),d1P(3),d3P(3)

    double precision pi,TwH,WrH,LkH,wrapprox
    integer nturns
    double precision eta,veta,m2,m3,u2,u3,psi
    DOUBLE PRECISION, EXTERNAL :: getpsi
!----------------------------------------------------------------------
! NOTE :
! Parameters set in this subroutine should be considered as ``solution
! measures'' and be used for output purposes only.
!
! They should never be used as `true'' continuation parameters.
!
! They may, however, be added as ``over-specified parameters'' in the
! parameter list associated with the AUTO-Constant NICP, in order to
! print their values on the screen and in the b.xxx file.
!
! They may also appear in the list associated with AUTO-constant NUZR.
!
!----------------------------------------------------------------------
! For algebraic problems the argument U is, as usual, the state vector.
! For differential equations the argument U represents the approximate
! solution on the entire interval [0,1]. In this case its values can
! be accessed indirectly by calls to GETP, as illustrated below, or
! by obtaining NDIM, NCOL, NTST via GETP and then dimensioning U as
! U(NDIM,0:NCOL*NTST) in a seperate subroutine that is called by PVLS.
!----------------------------------------------------------------------

pi=4*dATan(1.d0)

! --- On lit les PAR
CALL GETPAR(n20,n30,m20,m30,eta0,w,typepsi,typeconti,alphaL,Lk,Tw,Wr,eps,typeF18,PAR)
! FIN On lit les PAR

! --- Calcul de u2(0) et u3(0)
eta =GETP('BV0',17,U)
veta=GETP('BV0',18,U)
m2  =GETP('BV0',02,U)
m3  =GETP('BV0',03,U)
psi = getpsi(w,veta,typepsi) ! psi(w eta')
u2  = (m2 + eta * m3) / (1.d0 + eta**2)**2 / psi
u3  = eta * u2
PAR(13) = u2 ! u2(0)
PAR(14) = u3 ! u3(0)
! FIN Calcul de u2(0) et u3(0)

! --- Calcul de u2(0.5) et u3(0.5)
eta =GETP('BV1',17,U)
if (dabs(eta).gt.1e-9) then
  print*," eta(0.5) trop grand : eta(0.5) = ",eta
endif
veta=GETP('BV1',18,U)
m2  =GETP('BV1',02,U)
m3  =GETP('BV1',03,U)
psi = getpsi(w,veta,typepsi) ! psi(w eta')
u2  = (m2 + eta * m3) / ( (1.d0 + eta**2)**2 * psi)
u3  = eta * u2
PAR(15) = u2 ! u2(0.5)
PAR(16) = u3 ! u3(0.5)
! FIN Calcul de u2(0.5) et u3(0.5)

PAR(17) = GETP('BV1',2,U) ! m2(1/2)
PAR(18) = GETP('BV1',3,U) ! m3(1/2)
PAR(19) = GETP('BV1',18,U) ! veta(1/2)
PAR(20) = GETP('BV1',10,U) ! d2x(1/2)
PAR(21) = GETP('BV1',14,U) ! d3y(1/2)
PAR(22) = GETP('BV1',04,U) ! x(1/2)


! --- Calcul de Tw, Lk et Wr

! --- Calcul de Tw
TwH = 2.d0 * GETP('BV1',16,U) / (2.d0*pi)      ! integration sur la solution actuelle
PAR(11) = TwH  ! tw(TOTAL), on met a jour de toutes les facons
! FIN Calcul de Tw

if (typeconti.eq.0) then ! vary Lk (fixed : w, ...)
  ! --- calcul de alpha_here
  d1M(1)= GETP('BV1',07,U)  ! d1x(-1/2)
  d1M(2)=-GETP('BV1',08,U)  ! -d1y(-1/2)
  d1M(3)=-GETP('BV1',09,U)  ! -d1z(-1/2)

  d1P(1)=GETP('BV1',07,U)  ! U1(07)=d1x(1/2)
  d1P(2)=GETP('BV1',08,U)  ! d1y(1/2)
  d1P(3)=GETP('BV1',09,U)  ! d1z(1/2)

  d3P(1)=GETP('BV1',13,U)  ! d3x(1/2)
  d3P(2)=GETP('BV1',14,U)  ! U1(14)=d3y(1/2)
  d3P(3)=GETP('BV1',15,U)  ! d3z(1/2)

  call COMPAL(d1M,d1P,d3P,alphaH,1)  ! le resultat est dans 'alphaH'
  ! FIN calcul de alpha_here

  wrapprox = Wr        ! pris du PAR() c'est en fait le Wr du point d avant
  nturns = dnint(TwH - alphaH/(2.d0*pi) + wrapprox)
  LkH = nturns + (alphaH)/(2.d0*pi)
  WrH = LkH - TwH
  PAR(12) = WrH  ! tout comme le Tw, le Wr est mis a jour dans tous les cas
  PAR(09) = alphaH ! on met alpha a jour
  PAR(10) = LkH ! on met le link a jour
endif

if (typeconti.eq.1) then ! vary w ou eps (fixed : Lk, ...)
  ! tout comme le Tw, le Wr est mis a jour dans tous les cas
  ! mais ici, comme le Lk ne bouge pas, on prend celui dans le PAR()
  PAR(12) = Lk - TwH
endif

if (typeconti.eq.2) then ! vary (w ou eps) at Lk=1/2
  ! tout comme le Tw, le Wr est mis a jour dans tous les cas
  ! mais ici, comme le Lk ne bouge pas, on prend celui dans le PAR()
  PAR(12) = Lk - TwH
endif

! FIN Calcul de Tw, Lk et Wr



!      EXEMPLES EXEMPLES
!
!      print*, 'dans PVLS : ',numli
!      write(10,123)numli,npar,(par(i),i=1,npar)
! 123  format(I8,1x,I4,1x,100(e18.10,1x))

! Set PAR(2) equal to the L2-norm of U(1)
!       PAR(2)=GETP('NRM',1,U)

! Set PAR(3) equal to the minimum of U(2)
!       PAR(3)=GETP('MIN',2,U)

! Set PAR(4) equal to the value of U(2) at the left boundary.
!       PAR(4)=GETP('BV0',2,U)

! Set PAR(5) equal to the value of U(2) at the left boundary using
! another method
!       NDX=NINT(GETP('NDX',0,U))
!       NTST=NINT(GETP('NTST',0,U))
!       NCOL=NINT(GETP('NCOL',0,U))
!       PAR(5)=GETU2(U,NDX,NTST,NCOL)
!----------------------------------------------------------------------
! The first argument of GETP may be one of the following:
!        'NRM' (L2-norm),     'MAX' (maximum),
!        'INT' (integral),    'BV0 (left boundary value),
!        'MIN' (minimum),     'BV1' (right boundary value).
!        'MNT' (t value for minimum)
!        'MXT' (t value for maximum)
!        'NDIM', 'NDX' (effective (active) number of dimensions)
!        'NTST' (NTST from constant file)
!        'NCOL' (NCOL from constant file)
!        'NBC'  (active NBC)
!        'NINT' (active NINT)
!        'DTM'  (delta t for all t values, I=1...NTST)
!        'WINT' (integration weights used for interpolation, I=0...NCOL)
!
! Also available are
!   'STP' (Pseudo-arclength step size used).
!   'FLD' (`Fold function', which vanishes at folds).
!   'BIF' (`Bifurcation function', which vanishes at singular points).
!   'HBF' (`Hopf function'; which vanishes at Hopf points).
!   'SPB' ( Function which vanishes at secondary periodic bifurcations).
!   'EIG' ( Eigenvalues/multipliers, I=1...2*NDIM, alternates real/imag parts).
!   'STA' ( Number of stable eigenvalues/multipliers).
!----------------------------------------------------------------------

    END SUBROUTINE PVLS
!======================================================================



! !======================================================================
!     DOUBLE PRECISION FUNCTION GETU2(U,NDX,NTST,NCOL)
!     ! j utilise pas trop ce truc pour l instant
! !======================================================================
!     INTEGER, INTENT(IN) :: NDX,NCOL,NTST
!     DOUBLE PRECISION, INTENT(IN) :: U(NDX,0:NCOL*NTST)
! !----------------------------------------------------------------------
!     GETU2 = U(2,0)
!
!     END FUNCTION GETU2
! !======================================================================



!======================================================================
    SUBROUTINE STPNT(NDIM,U,PAR,T)
    ! par grand chose a modifier ici en general
!======================================================================

    IMPLICIT NONE
    INTEGER, INTENT(IN) :: NDIM
    DOUBLE PRECISION, INTENT(INOUT) :: U(NDIM), PAR(*)
    DOUBLE PRECISION, INTENT(IN) :: T

    integer i, npar
    double precision nparx
!----------------------------------------------------------------------
! en plus du fichier DAT, dont le nom est a mettre dans le fichier des
! constantes (c.xxx), il faut ici mettre les valeurs des parametres
! pour la config de depart
! --- On lit les parametres
    if (t .eq. 0) then ! apres les PAR restent en memoire
        print*," *-* Starting from PAR values instead of a solution"
        open(16,file="param_stpnt.txt")
        read(16,*) nparx
        npar=nparx
        read(16,*) (par(i),i=1,npar)
        close(16)
        print*,'voila les PAR lus : ',(par(i),i=1,npar)
    endif
! FIN On lit les parametres

    END SUBROUTINE STPNT
!======================================================================



    SUBROUTINE ICND
    END SUBROUTINE ICND

    SUBROUTINE FOPT
    END SUBROUTINE FOPT
