-- March 2022, moebius-wunderlich
-- we place s=0 at the flip sym. point, just to show that MERCI can go through the singular point

--require "Utils" -- unused

function drawFrame(pos, frame, scale, mesher)
  local ox=Arrow.new()
  ox.position=pos
  ox.direction={frame[1],frame[4],frame[7]}
  ox.color={255,0,0}
  ox.name="frameX"
  ox:setupDimensions(scale,0.3*scale,0.05*scale,0.1*scale)
  local oy=Arrow.new()
  oy.position=pos
  oy.direction={frame[2],frame[5],frame[8]}
  oy.color={0,255,0}
  oy.name="frameY"
  oy:setupDimensions(scale,0.3*scale,0.05*scale,0.1*scale)
  local oz=Arrow.new()
  oz.position=pos
  oz.direction={frame[3],frame[6],frame[9]}
  oz.color={0,0,255}
  oz.name="frameZ"
  oz:setupDimensions(scale,0.3*scale,0.05*scale,0.1*scale)
  mesher:add(ox)
  mesher:add(oy)
  mesher:add(oz)
end

function frameTheta(theta)
  -- read d1 d2 d3 in columns
      return {
              0, -math.sin(theta), math.cos(theta),
              1,               0,               0,
              0, math.cos(theta), math.sin(theta)}
end

function frameThetaPhi(theta,phi)
  -- read d1 d2 d3 in columns
    local sinTheta=math.sin(theta)
    local cosTheta=math.cos(theta)
    local sinPhi=math.sin(phi)
    local cosPhi=math.cos(phi)
    return {-sinPhi*sinTheta, -cosPhi*sinTheta, cosTheta,
              cosPhi,         -sinPhi,         0,
              sinPhi*cosTheta, cosPhi*cosTheta, sinTheta
    }
end

function genCurvFile(ribbon, filename)
  -- export the current state on the disk
  -- columns
  -- s Kappa Kappa' Eta Eta' x y z w ;
  -- 1  2      3     4   5   6 7 8 9 ;
  local w = ribbon.width
  --print()
  local file = io.open(filename, "w")
  --print(' Save file '..filename)
  local n = ribbon.numberSegments
  local l = 0

  for i=0,n-1,1 do
    local sys = ribbon:getMiniSys(i) -- a b n m : kappa = a s + b and eta = n s + m
    local p = ribbon:getPosStart(i) -- x y z
    file:write(l.."\t"..sys[2].."\t"..sys[1].."\t"..sys[4].."\t"..sys[3].."\t"..p[1].."\t"..p[2].."\t"..p[3].."\t"..w,"\n")
    l=l+ribbon:getLength(i)
  end

  local sys = ribbon:getMiniSys(n-1)
  local p = ribbon:endPos(n-1)
  local Lz = ribbon:getLength(n-1) -- length of last segment
  -- this is to obtain kappa(L) and eta(L)
  file:write(l.."\t"..(sys[1]*Lz+sys[2]).."\t"..sys[1].."\t"..(sys[3]*Lz+sys[4]).."\t"..sys[3].."\t"..(p[1]).."\t"..(p[2]).."\t"..(p[3]).."\t"..w,"\n")
  file:close()
end

function runExpe(name,ribbon,endPos,endFrame)
  -- simple clamp-clamp

  options = Options.new()
  options.maxIters = 1000
  options.checkDerivation = false
  options.tol = 1.e-9
  options.logLevel=3

  local exp = MRFitter.new() -- mixed ribbon (pas chaine donc)
  exp.name="test101"
  exp.gravity={0.,0.,0.}
  exp.ribbon=ribbon
  exp.options=options
  exp:attachEnd(endPos, endFrame) -- le bout s=L est clamped
  --exp:clampRuling(true, true) -- eta(0)=0 et eta(L)=0
  exp:declareEtaCyclic(true) -- eta(0)= - eta(L) -- could be done without, but not as neat
  --exp.oriIsConstraint=false -- The clamping at origin can be considered as a constraint (is the default behaviour), or the DOF and constraints can be removed from the system.
  exp:findEquilibrium() -- here we call IP-OPT
  print('Minim. '..name..'( '..ribbon.numberSegments..' segm.) '..exp.stats.status.." in "..exp.stats.iterations.." iterations with overall error = "..exp.stats.ktt_error)
  return exp.ribbon
end

function FromFlip(nseg,width)
  -- look for an equil. solution with s=0 at the flip point
  print()
  print(' = * = * - - - - - - - - - - - - - - - * = * = ')
  print()
  local sribbon
  ribbon = SuperRibbon.new()
  ribbon.width = width

  length = 1.0 -- adim
  ribbon.areaDensity = 0.0  -- no gravity
  ribbon.thickness = 0.0001 -- viewing purpose only
  ribbon.D = 1.0 -- adim
  ribbon.position = {0.0,0.0,0.0}  -- position of point s=0
  ribbon.frame = frameTheta(0.0) -- cosserat frame at s=0
  ribbon.poissonRatio=0.5 -- no use
  ribbon.numberSegments = nseg
  ribbon:setRibbonStart(2.0*math.pi,-1.0) -- kappa(s=0) and eta(s=0)

  -- c = 10 or 20
  -- eta(s) = Tanh[c*(s - 0.5)]*Cos[2*Pi*s]
  -- kappa(s) = - 2 pi Tanh[c*(s - 0.5)]
  for i=0,ribbon.numberSegments-1,1 do
    local si = i * length/ribbon.numberSegments -- arc length at start of segment i
    local coeff = 11.0 -- the bigger this coeff is, the bigger the variation at s=1/2
    kappaprime = - 2*math.pi*coeff / ( math.cosh( coeff*(si-0.5) ) )^2
    etaprime=coeff * math.cos(2*math.pi*si) / math.cosh( coeff * (si-0.5) )^2
    etaprime=etaprime -  2*math.pi * math.sin(2*math.pi*si) * math.tanh(coeff*(si-0.5))
    ribbon:setRibbonPiece(i,length/ribbon.numberSegments,kappaprime,etaprime) -- kappa'(si) and eta'(si)
    ribbon:setNaturalCurvature(i,0,0) -- no natural curvature
  end
  sribbon = MixedSuperRibbon.new(ribbon) -- Use the mix representation

  -- we export the configuration used for (aka seed) warm-start:
  genCurvFile(sribbon,"lst_z_eta_pos_width_"..string.format("%.5f",sribbon.width).."_seed.txt")

  mesher = Mesher.new("Moebius_start_at_flip.obj") -- use e.g. meshlab to open the obj file
  mesher.precision=0.005
  mesher:setColors({255,255,0},{0,255,255}) -- yellow / cyan
  mesher:add(sribbon) -- uncomment to draw the seed configuration (that is sent to the minimization algo)

  drawFrame({0.0,0.0,0.0}, frameTheta(0.0), 0.10, mesher) -- we draw the material frame at s=0
  sribbon=runExpe("Lk = 0.5", sribbon, {0,0,0}, frameThetaPhi(0,math.pi))

  print()

  print("After minimisation we have u1(s=0) = "..sribbon:getOmega1(0)[2])
  print("After minimisation we have u1(s =~ 1/2) = "..sribbon:getOmega1(math.floor(sribbon.numberSegments/2.0))[2])
  print("After minimisation we have eta(s=0) = "..sribbon:getEta(0)[2])
  print("After minimisation we have eta(s =~ 1/2) = "..sribbon:getEta(math.floor(sribbon.numberSegments/2.0))[2])

  -- we export the equilibrium found by IP-OPT
  genCurvFile(sribbon,"lst_z_eta_pos_width_"..string.format("%.5f",sribbon.width)..".txt")

  mesher:setColors({255,0,0},{0,0,255}) -- red / blue
  mesher:add(sribbon) -- we draw the final configuration

  print()
end


--FromFlip(nseg, width )

FromFlip(95, 0.08) -- Succeeded in 137 iterations with overall error = 9.1e-11
