require "Utils"

function frameTheta(theta)
  -- columns are d1, d2, d3 : the cosserat frame
      return {
              0, -math.sin(theta), math.cos(theta),
              1,               0,               0,
              0, math.cos(theta), math.sin(theta)}
    end

function frameThetaPhi(theta,phi)
    local sinTheta=math.sin(theta)
    local cosTheta=math.cos(theta)
    local sinPhi=math.sin(phi)
    local cosPhi=math.cos(phi)
    return {-sinPhi*sinTheta, -cosPhi*sinTheta, cosTheta,
              cosPhi,         -sinPhi,         0,
              sinPhi*cosTheta, cosPhi*cosTheta, sinTheta
    }
end

function SimpleContactDetector(ribbon, plane,mesher)
  --[[ Simple contact detection. If distance of between point and plane is negligible, we print a message and add a sphere in output OBJ.
       Points considered are points between segments at both sides of the ribbon.
      ]]
  local plane = makePlane(plane)
  local t = ribbon.numberSegments
  for i=0, t-1, 1 do
    local pos = Vector.new(ribbon:getPosStart(i))
    local frame = Frame.new(ribbon:getFrameStart(i))
    local eta = ribbon:getEta(i)[2]
    local g = frame:generatrix(eta)
    local extr1 = g:cpy()
    extr1:scale(0.5*ribbon.width)
    extr1=pos+extr1
    local extr2 = g:cpy()
    extr2:scale(-0.5*ribbon.width)
    extr2=pos+extr2
    local d1=plane:distance(extr1)
    local d2=plane:distance(extr2)
    if math.abs(d1) < 1.e-5 then
      print("::: Contract "..i..",1 ("..d1..")")
      mesher:addSphere(extr1,0.002,10,{255,0,0})
    end
    if math.abs(d2) < 1.e-5 then
      print("::: Contract "..i..",2 ("..d2..")")
      mesher:addSphere(extr2,0.002,10,{255,0,0})
    end
  end
end

function split(sribbon, cuts)
  --[[ Subdivide segments near ends of the ribbon (at Wunderlich event a priori)
      ]]
  if cuts == nil then cuts = 4 end
  local t = sribbon.numberSegments
  local ribbon = sribbon:copy()
  ribbon.numberSegments = t+(cuts-1)*6; --last 3 and first 3 segments are each subdivised in 'cuts' segments: these 6 segments are now 24 segments
  for i=0,2,1 do
    local nl = sribbon:getLength(i)/cuts
    local s = sribbon:getMiniSys(i)
    for j = 0,cuts-1,1 do
      ribbon:setLength(i*cuts+j, nl)
      ribbon:setMiniSys(i*cuts+j, s[1],  s[1]*nl*j+s[2], s[3], s[3]*nl*j+s[4])
    end
  end
  for i=3,t-4,1 do
    ribbon:setLength(i+3*(cuts-1), sribbon:getLength(i))
    ribbon:setMiniSys(i+3*(cuts-1), sribbon:getMiniSys(i))
  end
  for i=0,2,1 do
    local nl = sribbon:getLength(t-3+i)/cuts
    local s = sribbon:getMiniSys(t-3+i)
    for j = 0,cuts-1,1 do
      ribbon:setLength(t-3+3*(cuts-1)+i*cuts+j, nl)
      ribbon:setMiniSys(t-3+3*(cuts-1)+i*cuts+j, s[1],  s[1]*nl*j+s[2], s[3], s[3]*nl*j+s[4])
    end
  end
  ribbon:reChain()
  return ribbon
end

function vts(v)
  return (v[1]..", "..v[2]..", "..v[3])
end

function runExpe(name,ribbon,endPos,endFrame)
  --[[Clamp-clamp, set eta(l)=eta(0)=0
      Used to turn the end clamp
      ]]
  options = Options.new()
  options.maxIters = 1000
  options.checkDerivation = false
  options.tol = 1.e-9
  options.logLevel=1

  local exp = MRFitter.new()
  exp.name="test101"
  exp.gravity={0.,0.,0.}
  exp.ribbon=ribbon
  exp.options=options
  exp:attachEnd(endPos, endFrame)
  exp:clampRuling(true, true)

  print("::::: Step (Expe) = "..name)
  exp.oriIsConstraint=false
  exp:findEquilibrium()
  print("::::: "..exp.stats.status.." in "..exp.stats.iterations.." iterations".."with overall error = "..exp.stats.ktt_error)
  return exp.ribbon
end

function runExpe2(name,ribbon,pos)
  --[[Make circular,
      no gravity here
      Used to finalize moebius ribbon
      ]]
  options = Options.new()
  options.maxIters = 100
  options.checkDerivation = false
  options.tol = 1.e-9
  options.logLevel=0

  local exp = MRFitter.new()
  exp.name="test101"
  exp.gravity={0,0,0}
  exp.ribbon=ribbon
  exp.options=options
  exp:detachOrClamping() -- r(0) no longer fixed
  exp:declareRibbonCyclic(true) -- r(0)=r(L) and cosserat also
  exp:clampRuling(true, true) -- eta(L)=eta(0)=0
  exp:useLooseClamping(pos,0) -- r(0) in a sphere of radius 0, centered at 'pos'

  print("::::: Step (Expe2)"..name)
  exp.oriIsConstraint=false
  exp:findEquilibrium()
  print("::::: "..exp.stats.status.." in "..exp.stats.iterations.." iterations".."with overall error = "..exp.stats.ktt_error)
  return exp.ribbon
end

function runExpe3T(name,ribbon,pos,posmid,normalmid,vecG,gamma,mesher)
  --[[Set eta(l)=eta(0)=0,
      add a plane at the middle of the ribbon to keep symetries
      Used for the ribbon on the edge
      ]]
  options = Options.new()
  options.maxIters = 1000
  options.checkDerivation = false
  options.tol = 1.e-9
  options.logLevel=2

  local exp = MRFitter.new()
  exp.name="test101"
  exp.gravity=vecG  -- orientation of gravity (normed vector)
  ribbon.areaDensity=gamma
  exp.ribbon=ribbon
  exp.options=options
  exp:detachOrClamping() -- r(0) no longer fixed
  exp:declareRibbonCyclic(true) -- r(0)=r(L) and cosserat also
  exp:clampRuling(true, true) -- eta(L)=eta(0)=0
  --exp:declareEtaCyclic(true) -- eta(L) = - eta(0) -- works as well, but could shift the singular point s=s+delta s
  exp:addOffPlaneDoubleEnd(posmid,normalmid) -- we create and place the planar support
  exp:alignBarycenterOnAxis({0,0,0},vecG)  -- the ribbon center of mass has to stay on an axis normal to the planar support (which is along gravity vector)

  print("::::: Step (Expe3T)"..name)
  exp.oriIsConstraint=false
  --
  exp:findEquilibrium()
  print("::::: "..exp.stats.status.." in "..exp.stats.iterations.." iterations".."with overall error = "..exp.stats.ktt_error)
  print("::::: end of Exp3T r(s=0) = "..vts(exp.ribbon:getPosStart(0)))
  mesher:setColors({20,05,150},{255,0,0})
  mesher:add(exp.ribbon)
  SimpleContactDetector(exp.ribbon,{posmid,normalmid},mesher)
  --
  return exp.ribbon
end

function TwistProtocolTranche(gamma,nseg,w)
  -- We look for a ribbon sitting on its edge.
  print('* ===================  Heavy Moebius on rigid support =================== *')
  --
  local mesher = Mesher.new("MoebiusTable_w"..w.."G"..gamma.."nseg"..nseg..".obj")
  mesher.precision=0.005
  mesher:setColors({255,255,0},{0,255,255})
  --
  local sribbon
  ribbon = SuperRibbon.new()
  ribbon.width=w
  ribbon.thickness=0.0001
  length = 1.0 -- adim
  ribbon.areaDensity=0.0  -- we start with no gravity
  ribbon.D = 1.0 -- adim
  ribbon.position={0.,0.,0.} -- r(0)
  ribbon.frame=frameTheta(0.) -- cosserat(0)
  ribbon.poissonRatio=0.5 -- unused
  ribbon.numberSegments = nseg
  if math.abs((nseg/2)-math.floor(nseg/2)) > 0.1 then
    print('Error: we need an even number of segments!')
    print('Abort!')
    goto onsort
  end

  -- --- initialisation of the ribbon
  ribbon:setRibbonStart(-math.pi*2./length,0.01)
  for i=0,ribbon.numberSegments-1,1 do
      ribbon:setRibbonPiece(i,length/ribbon.numberSegments,0.0001,0.0001)
      ribbon:setNaturalCurvature(i,0,0)
  end
  -- END initialisation of the ribbon
  sribbon = MixedSuperRibbon.new(ribbon) -- Use the mix representation
  -- -- Turn end frame to twist ribbon
  for step=0,1,0.2 do  -- step=1 is moebius (Lk=1/2)
      sribbon=runExpe("Lk = "..(0.5*step), sribbon, {0,0,0}, frameThetaPhi(0,step*math.pi))
  end
  print("Post Exp1: u1(s=0) = "..sribbon:getOmega1(0)[2])
  print()

  sribbon = runExpe2("Make it cyclic",sribbon,{0,0,0})
  print("Post Exp2: u1(s=0) = "..sribbon:getOmega1(0)[2])
  print()

  local posmid = sribbon:getPosStart(sribbon.numberSegments/2) -- flip point: r(0.5)
  local framemid = sribbon:getFrameStart(sribbon.numberSegments/2)
  local normalmid = {framemid[2],framemid[5],framemid[8]} -- d2(0.5)
  print("We create a planar support at position: "..vts(posmid))
  print("with normal: "..vts(normalmid))
  plane = PlaneMesh.new(posmid,normalmid) -- support is created at bottom point of Moebius config with normal along d_2(s=0.5)
  plane.color={55,255,55}
  mesher:add(plane)
  myArrow=Arrow.new()
  myArrow.position=posmid
  myArrow.direction=normalmid
  myArrow.color={255,255,0}
  myArrow.name="arrow"
  scale=0.1
  myArrow:setupDimensions(scale,0.15*scale,0.025*scale,0.05*scale)
  mesher:add(myArrow)
  -- Nota Bene: gravity vector = - normal of support
  vecGrav = {-framemid[2],-framemid[5],-framemid[8]}  -- straight up
  --vecGrav = {0,-math.cos(0.6),math.sin(0.6)} -- if you want to make the ribbon fall on its side (more min steps, but should work)
  print("Gravity direction: "..vts(vecGrav))
  --if math.abs(Vector.norm(vecGrav)-1.0) > 0.001 then
  --  print("Error norm gravity vector = "..Vector.norm(vecGrav).." must be 1!")
  --end

  -- -- Now add plane and detach ribbon from clamp
  sribbon = runExpe3T("On table",sribbon,{0,0,0},posmid,normalmid,vecGrav,gamma,mesher)
  print("Post Exp3T: u1(s=0) = "..sribbon:getOmega1(0)[2])
  print()

  mesher:setColors({0,0,0},{255,255,205})
  drawFrame(sribbon.position, sribbon.frame, 0.04, mesher);
  mesher:addCenterline(sribbon)
  sribbon:genDiscreteDescriptor(0.1, "MoebiusTable_w"..w.."G"..gamma.."nseg"..nseg..".json") -- export shape and curvatures

  -- --- Compute distance from r(0) to plane
  local r0 = sribbon:getPosStart(0)
  print("Post Exp3T: r(s=0) = "..vts(r0))
  local planeF = makePlane({posmid,normalmid})
  H = planeF:distance(r0)
  print("H = "..H) -- Height of Moebius config (flip to singular point)
  -- END Compute distance from r(0) to plane

  -- --- We compute the position of the center of mass of the ribbon
  local rcdm = sribbon:getPosStart(0)
  nseg = sribbon.numberSegments
  for i=1,nseg-1,1 do
    ri = sribbon:getPosStart(i)
    tmp = Vector.add(rcdm,ri) -- rcdm=rcdm+sribbon:getPosStart(i)
    rcdm = tmp
  end
  Vector.scale(rcdm, 1.0/nseg) -- rcdm = rcdm/nseg
  Hcdm = planeF:distance(rcdm)
  print("Hcdm = "..Hcdm) -- height of center of mass
  -- END We compute the position of the center of mass of the ribbon

  -- --- Export some more information
  local out = io.output("MoebiusTable_w"..w.."G"..gamma.."nseg"..nseg..".txt")
  out:write(H .. "\n")
  out:write(w .. "\n")
  out:write(gamma .. "\n")
  out:write(Hcdm .. "\n")
  out:flush()
  out:close()
  -- END Export some information

  print("* We just did Gamma = "..gamma.." *")

  mesher:close()
  ::onsort::
end

--TwistProtocolTranche(gamma , nseg ,  w)
TwistProtocolTranche(28.0 , 80 , 0.025)
